import React from 'react';
import Facade from '../Datafacade/datafacade';
import { NavLink } from 'react-router-dom';
import Loader from 'react-loader-spinner';
import { usePromiseTracker } from "react-promise-tracker";
const LoadingIndicator = props => {
    const { promiseInProgress } = usePromiseTracker();
    return promiseInProgress &&
        <div
            style={{
                width: "100%",
                height: "100",
                display: "flex",
                justifyContent: "center",
                alignItems: "center"
            }}
        >
            <Loader type="TailSpin" color="#2BAD60" height="100" width="100" />
        </div>
};

class Leaderboard extends React.Component {
    constructor(props) {
        super(props)
        this.state = {
            players: [],
            breakload: true,
            scrollListener: {},
        };
        this.scrollListener = this.handleScroll.bind(this);
    }

    componentWillMount() {
        window.addEventListener('scroll', this.scrollListener);
    }

    componentWillUnmount() {
        window.removeEventListener("scroll", this.scrollListener);
    }

    componentDidMount = async () => {
        this.setState({ breakload: false })
        const players = await Facade.getLeaderBoardFromCache(0);
        this.setState({ breakload: true })
        this.setState({ players });
    };

    findSecondLast(array) {
        return array[array.length - 2];
    }

    handleScroll = () => {
        const player = this.findSecondLast(this.state.players);
        var lastLi = document.getElementById('leaderboard/' + player.steamID);
        var lastLiOffset = lastLi.offsetTop + lastLi.clientHeight;
        var pageOffset = window.pageYOffset + window.innerHeight;
        if (pageOffset > lastLiOffset && this.state.breakload) {
            this.setState({ breakload: false })
            this.loadPlayers();
        }
    };

    loadPlayers = async () => {
        const players = await Facade.getLeaderBoardFromCache(this.state.players.length + 1);
        this.setState({ players });
        this.setState({ breakload: true })
    }

    render() {
        return (
            <React.Fragment>
                <div className="alert alert-primary" >
                    <div id='containerMaps'>
                        <div>
                            <NavLink to={'/mapboard/'}>
                                <h3> <strong className="col-lg-1 col-centered" /> MapBoard<br /> </h3>
                            </NavLink>
                        </div>
                    </div>
                </div>
                <ul>
                    <div className="container mm" style={{ marginTop: "85px" }}>
                        <div style={{
                            position: 'absolute', left: '50%', top: '30%'
                        }}>
                            <br />
                            <br />
                            {this.state.players.map(e => (
                                <li key={e.steamID} id={"leaderboard/" + e.steamID}>
                                    <NavLink to={'/player/' + e.steamID}>
                                        <strong className="col-lg-1 col-centered" /> {e.steamID}<br />
                                        <strong className="col-lg-1 col-centered" />{e.name}<br />
                                        <img src={e.Avatar} alt="" /> <br />
                                        <strong className="col-lg-1 col-centered" />Rank: {e.Rank}<br />
                                        <strong className="col-lg-1 col-centered" />Points: {e.PlayerPoints}<br />
                                        <strong className="col-lg-1 col-centered" />Times: {e.Times}<br />
                                        <br />
                                        <br />
                                    </NavLink>
                                </li>
                            ))}
                            {!this.state.breakload ? <h1>
                                Loading Players <br /> <br /> <br /> <LoadingIndicator />
                            </h1> : ""}
                        </div>
                    </div>
                </ul>
            </React.Fragment>
        );
    }
}


export default Leaderboard;